import os
import threading
import base64
from tkinter import SE
from dotenv import load_dotenv
from google import genai
from google.genai import types

from HoloAI.HAIUtils.HAIUtils import (
    isStructured,
    formatTypedInput,
    formatTypedExtended,
    parseTypedInput,
    getFrames,
    supportsReasoning,
)

from HoloAI.HAIBaseConfig.BaseConfig import BaseConfig

load_dotenv()

class GoogleConfig(BaseConfig):
    def __init__(self, apiKey=None):
        super().__init__()
        self._setClient(apiKey)
        self._setModels()

    def _setClient(self, apiKey=None):
        if not apiKey:
            apiKey = os.getenv("GOOGLE_API_KEY")
        if not apiKey:
            raise KeyError("Google API key not found. Please set GOOGLE_API_KEY in your environment variables.")
        self.client = genai.Client(api_key=apiKey)

    def _setModels(self):
        self.RModel = os.getenv("GOOGLE_RESPONSE_MODEL", "gemini-2.5-flash")
        self.VModel = os.getenv("GOOGLE_VISION_MODEL", "gemini-2.5-flash")

    # ---------------------------------------------------------
    # Response
    # ---------------------------------------------------------
    def Response(self, **kwargs) -> str:
        model   = kwargs.get('model')
        system  = kwargs.get('system')
        user    = kwargs.get('user')  
        skills  = kwargs.get('skills')
        tools   = kwargs.get('tools')
        show    = kwargs.get('show')
        effort  = kwargs.get('effort')
        budget  = kwargs.get('budget')
        tokens  = kwargs.get('tokens')  
        verbose = kwargs.get('verbose')
        
        messages = parseTypedInput(user)

        configArgs = {
            "response_mime_type": "text/plain",
            "system_instruction": [system],
            "max_output_tokens": tokens,
        }
        if tools:
            configArgs["tools"] = tools

        if skills:
            additionalInfo = self.executeSkills(skills, user, tokens, verbose)
            if additionalInfo:
                messages.append(formatTypedInput("user", additionalInfo))

        # --- Only add reasoning config if the model supports it ---
        if supportsReasoning(model):
            if effort == "auto":
                budget = -1  # Auto budget
            configArgs["thinking_config"] = types.ThinkingConfig(thinking_budget=budget)

        generateConfig = types.GenerateContentConfig(**configArgs)

        args = self._getArgs(model, messages, generateConfig)

        response = self.client.models.generate_content(**args)
        return response if verbose else response.text

    # -----------------------------------------------------------------
    # Vision
    # -----------------------------------------------------------------
    def Vision(self, **kwargs):
        model   = kwargs.get('model')
        system  = kwargs.get('system')
        user    = kwargs.get('user')
        skills  = kwargs.get('skills')
        tools   = kwargs.get('tools')
        effort  = kwargs.get('effort')
        budget  = kwargs.get('budget')
        tokens  = kwargs.get('tokens')
        paths   = kwargs.get('paths')
        collect = kwargs.get('collect')
        verbose = kwargs.get('verbose')

        images = []
        for path in paths:
            frames = getFrames(path, collect)
            b64, mimeType, _ = frames[0]
            images.append(
                types.Part(
                    inline_data=types.Blob(
                        mime_type=f"image/{mimeType}",
                        data=base64.b64decode(b64)
                    )
                )
            )

        textPart = types.Part(text=user)
        messages = [ types.Content(role="user", parts=images + [textPart]) ]

        configArgs = {
            "response_mime_type": "text/plain",
            "system_instruction": [system],
            "max_output_tokens": tokens,
        }

        generateConfig = types.GenerateContentConfig(**configArgs)

        args = self._getArgs(model, messages, generateConfig)

        response = self.client.models.generate_content(**args)
        return response if verbose else response.text

    def processSkills(self, instructions, user, tokens) -> str:
        intent = self.extractIntent(user)
        messages = [formatTypedInput("user", intent)]
        configArgs = {
            "response_mime_type": "text/plain",
            "system_instruction": [instructions],
            "max_output_tokens": tokens,
        }
        generateConfig = types.GenerateContentConfig(**configArgs)
        args = self._getArgs(self.RModel, messages, generateConfig)
        response = self.client.models.generate_content(**args)
        return response.text

    def _getArgs(self, model, messages, config):
        args = {
            "model": model,
            "contents": messages,
            "config": config
        }
        return args




# OpenAIConfig.py in The HoloAI project is a configuration class for interacting with the OpenAI API.

import os
import threading
from dotenv import load_dotenv
from openai import OpenAI

from HoloAI.HAIUtils.HAIUtils import (
    isStructured,
    formatJsonInput,
    formatJsonExtended,
    parseJsonInput,
    getFrames,
    supportsReasoning,
)

from HoloAI.HAIBaseConfig.BaseConfig import BaseConfig

load_dotenv()

class OpenAIConfig(BaseConfig):
    def __init__(self, apiKey=None):
        super().__init__()
        self._setClient(apiKey)
        self._setModels()

    def _setClient(self, apiKey=None):
        if not apiKey:
            apiKey = os.getenv("OPENAI_API_KEY")
        if not apiKey:
            raise KeyError("OpenAI API key not found. Please set OPENAI_API_KEY in your environment variables.")
        self.client = OpenAI(api_key=apiKey)

    def _setModels(self):
        self.RModel = os.getenv("OPENAI_RESPONSE_MODEL", "gpt-4.1-mini")
        self.VModel = os.getenv("OPENAI_VISION_MODEL", "gpt-4.1")

    # ---------------------------------------------------------
    # Response
    # ---------------------------------------------------------
    def Response(self, **kwargs) -> str:
        model   = kwargs.get('model')
        system  = kwargs.get('system')
        user    = kwargs.get('user')
        skills  = kwargs.get('skills')
        tools   = kwargs.get('tools')
        show    = kwargs.get('show')
        effort  = kwargs.get('effort')
        budget  = kwargs.get('budget')
        tokens  = kwargs.get('tokens')
        verbose = kwargs.get('verbose')

        messages = []

        # --- system / instructions ---
        messages.append(formatJsonInput("system", system))

        # --- user memories / latest ---
        messages.extend(parseJsonInput(user))

        args = self._getArgs(model, messages, tokens)

        if tools:
            args["tools"] = tools

        if skills:
            additionalInfo = self.executeSkills(skills, user, tokens, verbose)
            if additionalInfo:
                messages.append(formatJsonInput("user", additionalInfo))

        # -- Only add reasoning config if the model supports it --
        if supportsReasoning(model):
            if effort == "auto":
                effort = "low"
            args["reasoning"] = {"effort": effort}
            if budget:
                args["max_output_tokens"] = budget

        response = self.client.responses.create(**args)
        return response if verbose else response.output_text

    # ---------------------------------------------------------
    # Vision
    # ---------------------------------------------------------
    def Vision(self, **kwargs):
        model   = kwargs.get('model')
        system  = kwargs.get('system')
        user    = kwargs.get('user')
        skills  = kwargs.get('skills')
        tools   = kwargs.get('tools')
        effort  = kwargs.get('effort')
        budget  = kwargs.get('budget')
        tokens  = kwargs.get('tokens')
        paths   = kwargs.get('paths')
        collect = kwargs.get('collect')
        verbose = kwargs.get('verbose')

        messages = []
        messages.append(formatJsonInput("system", system))

        images = []
        for path in paths:
            frames = getFrames(path, collect)
            b64, mimeType, idx = frames[0]
            images.append({
                "type": "input_image",
                "image_url": f"data:image/{mimeType};base64,{b64}"
            })

        userContent = [{"type": "input_text", "text": user}] + images
        payload = messages.copy()
        payload.append({
            "role": "user",
            "content": userContent
        })

        args = self._getArgs(model, payload, tokens)

        response = self.client.responses.create(**args)
        return response if verbose else response.output_text

    def processSkills(self, instructions, user, tokens) -> str:
        messages = []
        messages.append(formatJsonInput("system", instructions))
        intent = self.extractIntent(user)
        messages.append(formatJsonInput("user", intent))
        args = self._getArgs(self.RModel, messages, tokens)
        response= self.client.responses.create(**args).output_text
        return response

    def _getArgs(self, model, messages, tokens):
        args = {
            "model": model,
            "input": messages,
            "max_output_tokens": tokens,
        }
        return args



import os
import threading
from dotenv import load_dotenv
import anthropic

from HoloAI.HAIUtils.HAIUtils import (
    isStructured,
    formatJsonInput,
    formatJsonExtended,
    parseJsonInput,
    getFrames,
    supportsReasoning,
)

from HoloAI.HAIBaseConfig.BaseConfig import BaseConfig

load_dotenv()

class AnthropicConfig(BaseConfig):
    def __init__(self, apiKey=None):
        super().__init__()
        self._setClient(apiKey)
        self._setModels()

    def _setClient(self, apiKey=None):
        if not apiKey:
            apiKey = os.getenv("ANTHROPIC_API_KEY")
        if not apiKey:
            raise KeyError("Anthropic API key not found. Please set ANTHROPIC_API_KEY in your environment variables.")
        self.client = anthropic.Anthropic(api_key=apiKey)

    def _setModels(self):
        self.RModel = os.getenv("ANTHROPIC_TEXT_MODEL", "claude-3-5-haiku-latest")
        self.VModel = os.getenv("ANTHROPIC_VISION_MODEL", "claude-sonnet-4")

    # ---------------------------------------------------------
    # Response
    # ---------------------------------------------------------
    def Response(self, **kwargs) -> str:
        model   = kwargs.get('model')
        system  = kwargs.get('system')
        user    = kwargs.get('user')
        skills  = kwargs.get('skills')
        tools   = kwargs.get('tools')
        show    = kwargs.get('show')
        effort  = kwargs.get('effort')
        budget  = kwargs.get('budget')
        tokens  = kwargs.get('tokens')
        verbose = kwargs.get('verbose')

        # --- user memories / latest ---
        messages = parseJsonInput(user)

        args = self._getArgs(model, system, messages, tokens)

        if tools:
            args["tools"] = tools

        if skills:
            additionalInfo = self.executeSkills(skills, user, tokens, verbose)
            if additionalInfo:
                messages.append(formatJsonInput("user", additionalInfo))

        if supportsReasoning(model):
            if effort == "auto":
                budget = 1024
            # the budget can be no less than 1024 tokens so we need to ensure that
            if budget < 1024:
                orgBudget = budget
                budget = 1024
                print(f"[Notice] Budget ({orgBudget}) is less than 1024 tokens. Adjusting budget to {budget}.")
                print(f"[Notice] To avoid this, set budget to at least 1024 tokens.")
            # now we to make sure the token amount is greater than the budget
            if tokens and tokens < budget:
                orgTokens = tokens
                tokens = tokens + budget
                print(f"[Notice] Tokens ({orgTokens}) is less than budget ({budget}). Adjusting tokens to {tokens}.")
                print(f"[Notice] To avoid this, set tokens higher than your budget amount e.g tokens=1500, budget=1024")
                args["max_tokens"] = tokens
            args["thinking"] = {"type": "enabled", "budget_tokens": budget}
        
        response = self.client.messages.create(**args)
        if supportsReasoning(model):
            return response if verbose else next((block.text for block in response.content if getattr(block, "type", None) == "text"), "")
        return response if verbose else response.content[0].text

    # ---------------------------------------------------------
    # Vision
    # ---------------------------------------------------------
    def Vision(self, **kwargs):
        model   = kwargs.get('model')
        system  = kwargs.get('system')
        user    = kwargs.get('user')
        skills  = kwargs.get('skills')
        tools   = kwargs.get('tools')
        effort  = kwargs.get('effort')
        budget  = kwargs.get('budget')
        tokens  = kwargs.get('tokens')
        paths   = kwargs.get('paths')
        collect = kwargs.get('collect')
        verbose = kwargs.get('verbose')

        images = []
        for path in paths:
            frames = getFrames(path, collect)
            for b64, mimeType, idx in frames:
                images.append({
                    "type": "image",
                    "source": {
                        "type": "base64",
                        "media_type": f"image/{mimeType}",
                        "data": b64
                    }
                })

        userContent = images.copy()
        if user:
            userContent.append({
                "type": "text",
                "text": user
            })
        messages = [{
            "role": "user",
            "content": userContent
        }]

        args = self._getArgs(model, system, messages, tokens)

        response = self.client.messages.create(**args)
        return response if verbose else response.content[0].text

    def processSkills(self, instructions, user, tokens) -> str:
        messages = []
        intent = self.extractIntent(user)
        messages.append(formatJsonInput("user", intent))
        args = self._getArgs(self.RModel, instructions, messages, tokens)
        response = self.client.messages.create(**args)
        return response.content[0].text

    def _getArgs(self, model, system, messages, tokens):
        args = {
            "model": model,
            "system": system,
            "messages": messages,
            "max_tokens": tokens,
        }
        return args



import os
import threading
from SkillLink import SkillLink

from HoloAI.HAIUtils.HAIUtils import (
    parseInstructions,
    validateResponseArgs,
    validateVisionArgs,
)


class BaseConfig:
    _instance = None
    _lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            with cls._lock:
                if not cls._instance:
                    cls._instance = super(BaseConfig, cls).__new__(cls)
        return cls._instance

    def __init__(self):
        if getattr(self, 'initialized', False):
            return

        self.skillLink = SkillLink()  # Initialize the SkillLink instance
        self.skills  = None  # Default skills
        self.tools   = None  # Default tools
        self.show    = 'hidden'  # Default visibility for Reasoning/Thinking
        self.effort  = 'auto'  # Default effort level
        self.budget  = 1369  # Default budget
        self.tokens  = 3369  # Default tokens
        self.collect = 10  # Default frames to collect
        self.paths   = []  # Default paths for images
        self.collect = 10  # Default number of frames to collect
        self.verbose = False  # Default verbose mode

        self.initialized = True

    # ---------------------------------------------------------
    # Public methods
    # ---------------------------------------------------------
    def getResponse(self, **kwargs):
        """
        Get a Response from the configured model.
        :param kwargs: Keyword arguments to customize the request.
            - model: (str) The model to use (Required).
            - system/instructions: (str) System prompt or additional instructions (Optional).
            - user/input: (str or list) The main user input (Required). 
                Accepts a single prompt string or a message history (list of messages). 
                Both 'user' and 'input' are interchangeable; use either (Preferred: input).
            
            - tools: (list) Tools to use (Optional).
            - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
            - show: (str) Show reasoning/thinking ('hidden', 'parsed', 'raw') (Optional (default: 'hidden')).
            - effort: (str) Effort level ('auto', 'low', 'medium', 'high') (Optional (default: 'auto')).
            - budget/max_budget: (int) Budget for the response (Optional (default: 1369)).
            - verbose: (bool) Return verbose output (Optional (default: False)).
        :return: A Response object.
        """
        model   = kwargs.get('model')
        system  = parseInstructions(kwargs)
        user    = kwargs.get('user') or kwargs.get('input')
        skills  = kwargs.get('skills', self.skills)
        tools   = kwargs.get('tools', self.tools)
        show    = kwargs.get('show', self.show)
        effort  = kwargs.get('effort', self.effort)
        budget  = kwargs.get('budget', self.budget) or kwargs.get('max_budget', self.budget)
        tokens  = kwargs.get('tokens', self.tokens) or kwargs.get('max_tokens', self.tokens)
        paths   = kwargs.get('paths', self.paths)
        collect = kwargs.get('collect', self.collect)
        verbose = kwargs.get('verbose', self.verbose)
        validateResponseArgs(model, user)
        return self.Response(model=model, system=system, user=user, skills=skills, tools=tools, show=show, effort=effort, budget=budget, tokens=tokens, verbose=verbose)

    def getVision(self, **kwargs):
        """
        Get a Vision response from the configured model.
        :param kwargs: Keyword arguments to customize the request.
            - model: (str) The model to use (Required).
            - system/instructions: (str) System prompt or additional instructions (Optional).
            - user/input: (str or list) The main user input (Required). 
                Accepts a single prompt string or a message history (list of messages). 
                Both 'user' and 'input' are interchangeable; use either (Preferred: input).
            - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
            - paths: (list) List of image paths (default: empty list).
            - collect: (int) Number of frames to collect (default: 10).
            - verbose: (bool) Return verbose output (Optional (default: False)).
        :return: A Vision response object.
        """
        model   = kwargs.get('model')
        system  = parseInstructions(kwargs)
        user    = kwargs.get('user') or kwargs.get('input')
        skills  = kwargs.get('skills', self.skills)
        tools   = kwargs.get('tools', self.tools)
        show    = kwargs.get('show', self.show)
        effort  = kwargs.get('effort', self.effort)
        budget  = kwargs.get('budget', self.budget) or kwargs.get('max_budget', self.budget)
        tokens  = kwargs.get('tokens', self.tokens) or kwargs.get('max_tokens', self.tokens)
        paths   = kwargs.get('paths', self.paths)
        collect = kwargs.get('collect', self.collect)
        verbose = kwargs.get('verbose', self.verbose)
        validateVisionArgs(model, user, paths)
        return self.Vision(model=model, system=system, user=user, paths=paths, collect=collect, tokens=tokens, verbose=verbose)

    def skillInstructions(self, capabilities):
        """
        Get skill instructions for the ava based on its capabilities.
        NOTE: the skillInstructions in the skillLink method will automatically use your naming conventions you can also,
        - pass limit=(int e.g 10) to limit the number of examples included in the instructions, or
        - pass verbose=True to view the instructions in detail as it will print the instructions to the console.
        """
        return self.skillLink.skillInstructions(capabilities)

    def getActions(self, action: str) -> list:
        """
        Get a list of actions based on the given action string.
        This method uses the skills manager's action parser to retrieve actions that match the given string.
        If the action is not found, it returns an empty list.
        """
        return self.skillLink.actionParser.getActions(action)

    def executeActions(self, actions, action):
        """
        Execute a list of actions using the skillLink's executeActions method.
        This method will return the results of the executed actions.
        If the actions are not found, it will return an empty list.
        """
        return self.skillLink.executeActions(actions, action)

    def executeSkills(self, skills, user, tokens, verbose=False) -> str:
        """
        Execute skills based on the provided skills, user input, and tokens.
        This method processes the skills, retrieves the actions, and executes them.
        :param skills: List of skills to execute.
        :param user: User input to provide context for the skills.
        :param tokens: Number of tokens to use for the skills execution.
        :param verbose: If True, prints detailed information about the execution.
        :return: A string containing the results of the executed skills.
        """
        if skills:
            agentSkills, actions = (skills or [None, None])[:2]
            instructions  = self.skillInstructions(agentSkills)
            calledActions = self.processSkills(instructions, user, tokens)
            getActions    = self.getActions(calledActions)
            if getActions:
                results         = self.executeActions(actions, getActions)
                filteredResults = [str(result) for result in results if result]
                if filteredResults:
                    combined = "\n".join(filteredResults)
                    if verbose:
                        print(f"Combined Results:\n{combined}\n")
                    return f"Use these results from the actions called when responding:\n{combined}"
        return 'None'

    def extractIntent(self, user):
        """
        Extract the last user text from a string, a list of message dicts, or a mixed list.
        Returns a string suitable for use in skills or message formatting.
        """
        if isinstance(user, list) and user:
            lastMsg = user[-1]
            # If message dict (e.g., {"role": "user", "content": ...})
            if isinstance(lastMsg, dict):
                content = lastMsg.get("content", "")
                # If content is a list of type-objects (OpenAI v1)
                if isinstance(content, list):
                    # Extract all "text" from type="text" parts, join by space
                    return " ".join(
                        str(part.get("text", "")) for part in content if part.get("type") == "text"
                    ).strip()
                # If content is a string (Anthropic etc.)
                if isinstance(content, str):
                    return content.strip()
                # Fallback: convert to string
                return str(content).strip()
            # If it's a string, just return
            if isinstance(lastMsg, str):
                return lastMsg.strip()
            # Fallback for unexpected
            return str(lastMsg).strip()
        elif isinstance(user, str):
            return user.strip()
        return ""
